@@ -45,34 +45,46 @@
 #include <errno.h>
 #include <ctype.h>
 
+// Determine the compiler (hopefully) and define conveniently named macros
+// accordingly.
+#if   defined(__ICC) || defined(__INTEL_COMPILER)
+  #define BLIS_ICC
+#elif defined(__clang__)
+  #define BLIS_CLANG
+#elif defined(__GNUC__)
+  #define BLIS_GCC
+#endif
+
 // Determine if we are on a 64-bit or 32-bit architecture.
 #if defined(_M_X64) || defined(__x86_64) || defined(__aarch64__) || \
     defined(_ARCH_PPC64)
-#define BLIS_ARCH_64
+  #define BLIS_ARCH_64
 #else
-#define BLIS_ARCH_32
+  #define BLIS_ARCH_32
 #endif
 
 // Determine the target operating system.
 #if defined(_WIN32) || defined(__CYGWIN__)
-#define BLIS_OS_WINDOWS 1
+  #define BLIS_OS_WINDOWS 1
+#elif defined(__gnu_hurd__)
+  #define BLIS_OS_GNU 1
 #elif defined(__APPLE__) || defined(__MACH__)
-#define BLIS_OS_OSX 1
+  #define BLIS_OS_OSX 1
 #elif defined(__ANDROID__)
-#define BLIS_OS_ANDROID 1
+  #define BLIS_OS_ANDROID 1
 #elif defined(__linux__)
-#define BLIS_OS_LINUX 1
+  #define BLIS_OS_LINUX 1
 #elif defined(__bgq__)
-#define BLIS_OS_BGQ 1
+  #define BLIS_OS_BGQ 1
 #elif defined(__bg__)
-#define BLIS_OS_BGP 1
+  #define BLIS_OS_BGP 1
 #elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || \
-      defined(__bsdi__) || defined(__DragonFly__)
-#define BLIS_OS_BSD 1
+      defined(__bsdi__) || defined(__DragonFly__) || defined(__FreeBSD_kernel__)
+  #define BLIS_OS_BSD 1
 #elif defined(EMSCRIPTEN)
-#define BLIS_OS_EMSCRIPTEN
+  #define BLIS_OS_EMSCRIPTEN
 #else
-#error "Cannot determine operating system"
+  #error "Cannot determine operating system"
 #endif
 
 // A few changes that may be necessary in Windows environments.
@@ -84,11 +96,11 @@
   #include <windows.h>
 
   #if !defined(__clang__) && !defined(__GNUC__)
-     // Undefine attribute specifiers in Windows.
-     #define __attribute__(x)
+    // Undefine attribute specifiers in Windows.
+    #define __attribute__(x)
 
-     // Undefine restrict.
-     #define restrict
+    // Undefine restrict.
+    #define restrict
   #endif
 
 #endif
